@@ -2,11 +2,16 @@ catSDML <- function(...) cat(..., append=TRUE, sep="")
 
 etagsSDML <- function(x, ...)
 {
+  info <- attr(x, "info")
   for (i in 1:length(x)) {
+    inf <- if (!is.null(info) && !is.na(info[i]))
+      paste(" info=\"", info[i], "\"", sep="")
+    else ""
+    
     if (is.na(x[i]) && !is.nan(x[i]))
-      catSDML("\<na/>", ...)
+      catSDML("\<na", inf, "/>", ...)
     else
-      tags(x[i], "e", ...)
+      tags(x[i], "e", inf, ...)
     
     if((i!=length(x)) & ((i %% 5)==0))
       catSDML("\n", ...)
@@ -15,11 +20,16 @@ etagsSDML <- function(x, ...)
 
 cetagsSDML <- function(x, ...)
 {
+  info <- attr(x, "info")
   for (i in 1:length(x)) {
+    inf <- if (!is.null(info) && !is.na(info[i]))
+      paste(" info=\"", info[i], "\"", sep="")
+    else ""
+    
     if (is.na(x[i]) && !is.nan(x[i]))
-      catSDML("\<na/>", ...)
+      catSDML("\<na",inf,"/>", ...)
     else {
-      catSDML("\<ce>", ...)
+      catSDML("\<ce",inf,">", ...)
       tags(Re(x[i]),"r", ...)
       tags(Im(x[i]),"i", ...)
       catSDML("\</ce>", ...)
@@ -30,15 +40,15 @@ cetagsSDML <- function(x, ...)
   }
 }
 
-tags <- function(x, s, ...) {
+tags <- function(x, s, info = "", ...) {
   if (is.nan(x))
-    catSDML("\<",s,"><nan/></",s,">", ...)
+    catSDML("\<", s, info, "><nan/></", s, ">", ...)
   else if (x == Inf)
-    catSDML("\<",s,"><posinf/></",s,">", ...)
+    catSDML("\<", s, info, "><posinf/></", s, ">", ...)
   else if (x == -Inf)
-    catSDML("\<",s,"><neginf/></",s,">", ...)
+    catSDML("\<", s, info, "><neginf/></", s, ">", ...)
   else
-    catSDML("\<",s,">", x, "\</",s,">", ...)
+    catSDML("\<", s, info, ">", x, "\</", s, ">", ...)
 }
 
 getAttrSDML <- function(x)
